@@ -64,7 +64,7 @@ def get_sample_sheet_s3(zip_reader):
     raise FileNotFoundError('Could not find sample sheet in s3 file.')
 
 
-def find_sample_sheet(dir_path):
+def find_sample_sheet(dir_path, return_all=False):
     """Find sample sheet file for Illumina methylation array.
 
     Notes:
@@ -75,6 +75,8 @@ def find_sample_sheet(dir_path):
 
     Arguments:
         dir_path {string or path-like} -- Base directory of the sample sheet and associated IDAT files.
+        return_all -- if True,
+            returns a list of paths to samplesheets, if multiple present, instead of raising an error.
 
     Raises:
         FileNotFoundError: [description]
@@ -121,7 +123,10 @@ def find_sample_sheet(dir_path):
         if len(name_matched) == 1:
             pass
         else:
-            raise Exception('Too many sample sheets in this directory. Move or rename redundant ones. Or specify the path to the one to use with --sample_sheet')
+            if return_all:
+                return name_matched
+            else:
+                raise Exception('Too many sample sheets in this directory. Move or rename redundant ones. Or specify the path to the one to use with --sample_sheet')
 
     sample_sheet_file = candidates[0]
     LOGGER.debug('Found sample sheet file: %s', sample_sheet_file)
